/* adc.c - Autros Drawer Controller for the *** 16C64A *** */
/* v 2.0 */
 

#include <adc.h>  

#if 0
---------------------------------------------------------------------------
KCF 08-09-99 v2114 -
     added autoconfig support code
KCF 09-09-99 v2115 -
     fixed bin LED/REED pair ordering problem 
KCF 15-09-99 v2216 -
     -74A version
---------------------------------------------------------------------------
#endif

#ID 0x2216  /* controller/firmware revision ID */

/* ---- ID format ----------------------------------------------------------
#ID 'a''b''c''d' where
   'a' is Target Board ID -  1 = AMC-01: Autros Main Controller
                             2 = ADC-01: Autros Drawer Controller
                             3 = AKPC: Autros Keypad Controller
   'b' is Target Board Revision:  A,B,C, etc.
   'c' is Firmware Major Version Number
   'd' is Firmware Minor Version Number
---------------------------------------------------------------------------*/


#define DEBOUNCE_DELAY  20

struct bin_status
{
    byte data0;
    byte data1; 
    byte data2;
} bstat[2] = {{0,0,0},{0,0,0}};

byte command;
boolean led_on;

typedef struct  // Define an 8-bit byte as 8 bits
{
    short b0;
    short b1;
    short b2;
    short b3;
    short b4;
    short b5;
    short b6;
    short b7;
} bits;

bits SSPBUF, SSPCON, SSPSTAT, PIR1;

#byte SSPBUF = 0x13
#byte SSPCON = 0x14
#byte SSPSTAT= 0x94
#byte PIR1   = 0x0c
#byte SSPADD = 0x93

byte i2cAddress=0x00;

#separate
void initialize()
{
    set_tris_c(0x5f);   // all bits out except C3,C4 (I2C clock,data)
    set_tris_a(0x00);   // all bits are outputs
    set_tris_b(0x00);   // all bits are outputs
    set_tris_d(0x00);   // all bits are outputs
    set_tris_e(0x04);   // all bits are outputs except re2 (amc request line)
    port_a = 0x3f;      //
    port_b = 0xff;
    port_d = 0xff;
    port_e = 0x07;
    port_c = 0x1f;
}
 

bits switchByte;

#separate
void raw_bin_status(byte b)
{
    switchByte.b0 = input(PIN_A4);
    switchByte.b1 = input(PIN_A5);
    switchByte.b2 = input(PIN_E0);
    switchByte.b3 = input(PIN_E1);
    switchByte.b4 = input(PIN_A0);
    switchByte.b5 = input(PIN_A1);
    switchByte.b6 = input(PIN_A2);
    switchByte.b7 = input(PIN_A3);
    bstat[b].data0 = switchByte;

    switchByte.b0 = input(PIN_D4);
    switchByte.b1 = input(PIN_D5);
    switchByte.b2 = input(PIN_D6);
    switchByte.b3 = input(PIN_D7);
    switchByte.b4 = input(PIN_D0);
    switchByte.b5 = input(PIN_D1);
    switchByte.b6 = input(PIN_D2);
    switchByte.b7 = input(PIN_D3);
    bstat[b].data1 = switchByte;

    switchByte.b0 = input(PIN_B4);
    switchByte.b1 = input(PIN_B5);
    switchByte.b2 = input(PIN_B6);
    switchByte.b3 = input(PIN_B7);
    switchByte.b4 = input(PIN_B0);
    switchByte.b5 = input(PIN_B1);
    switchByte.b6 = input(PIN_B2);
    switchByte.b7 = input(PIN_B3);
    bstat[b].data2 = switchByte;

}

#separate
boolean status_changed()
{
    return  (bstat[0].data0 != bstat[1].data0) ||
            (bstat[0].data1 != bstat[1].data1) ||
            (bstat[0].data2 != bstat[1].data2);
}

byte previous = 0, portCvalue;
#separate
void get_bin_status()
{
    set_tris_a(0x3f);           // all 6 bits are inputs
    set_tris_e(0x07);           // bits 0,1, 2 are inputs
    set_tris_b(0xff);           // all bits are inputs
    set_tris_d(0xff);           // all bits are inputs
    delay_us(10);
    output_low(PIN_C7);
    raw_bin_status(previous = !previous);
    do // debounce loop
    {
        delay_ms(DEBOUNCE_DELAY);
        raw_bin_status(previous = !previous);
    } while ( status_changed() );
}

#separate
void send_bin_status()
{
    get_bin_status();
    i2c_write(bstat[0].data0);
    delay_ms(11);
    i2c_write(bstat[0].data1);
    delay_ms(11);
    i2c_write(bstat[0].data2);
    delay_ms(20);
    initialize();
}

#separate
void light_led(byte led)
{
    port_a |= 0x3f;
    port_b |= 0xff;
    port_d |= 0xff;
    port_e |= 0x03;

    switch (led)
    {
      case 1: output_low(PIN_A4); return;
      case 2: output_low(PIN_A5); return;
      case 3: output_low(PIN_E0); return;
      case 4: output_low(PIN_E1); return;
      case 5: output_low(PIN_A0); return;
      case 6: output_low(PIN_A1); return;
      case 7: output_low(PIN_A2); return;
      case 8: output_low(PIN_A3); return;

      case 9: output_low(PIN_D4); return;
      case 10: output_low(PIN_D5); return;
      case 11: output_low(PIN_D6); return;
      case 12: output_low(PIN_D7); return;
      case 13: output_low(PIN_D0); return;
      case 14: output_low(PIN_D1); return;
      case 15: output_low(PIN_D2); return;
      case 16: output_low(PIN_D3); return;

      case 17: output_low(PIN_B4); return;
      case 18: output_low(PIN_B5); return;
      case 19: output_low(PIN_B6); return;
      case 20: output_low(PIN_B7); return;
      case 21: output_low(PIN_B0); return;
      case 22: output_low(PIN_B1); return;
      case 23: output_low(PIN_B2); return;
      case 24: output_low(PIN_B3); return;
    }
}

boolean sending;
#int_ssp // I2C RECEIVED BYTE INTERRUPT
void i2c_isr()
{
    if (i2c_poll())
    {
	     command = i2c_read();
        if ( led_on )
        {
            output_high(PIN_C5);
            led_on = FALSE;
        }
        else
        {
            output_low(PIN_C5);             // light on-board LED
            led_on = TRUE;
        }
        if ( command == 0xff )
            sending = TRUE;
        else
            light_led(command);
    }
    else if ( sending )
    {
        send_bin_status();
        SSPCON.b6 = 0;  // clear buffer overflow bit
        SSPCON.b5 = 1;  // make sure ssp interrupt is enabled
        PIR1.b3 = 0;    // clear interrupt flag to reset for next command
        sending = FALSE;
    }
}

ADC_AutoConfig()
{
  i2cAddress= 0x20  + ((~port_c & 0x07)*2);
  SSPADD = i2cAddress;
}

signed long loopCounter=0, on=1;
void
main()
{
byte bin;

    
    initialize();
    ADC_AutoConfig();
    for ( bin = 1; bin < 25; bin++ )
    {
        light_led(bin);
        delay_ms(100);
    }
    output_low(PIN_C5);             // light on-board LED
    led_on = TRUE;
    enable_interrupts(INT_SSP);
    enable_interrupts(GLOBAL);
    sending = FALSE;
    while (TRUE)
    {
    }
}
