#if 0
-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

MODULE: akc_kpad.c

PROJECT: Autros Keypad Controller firmware

DESCRIPTION:
         keypad module: keypad debouncing, encoding, audio alerts
         and keypad buffer accumulator

AUTHOR:  Kenneth C. Finney (KCF), Wilkes Associates, Inc.
         Gordon Wilkes, Autros Hospital Systems, Inc.

Copyright 1999. All Rights Reserved.

----------------------------------------------------------------------------

 See akc_main.c for Revisions & History

-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#endif

#include <pic.h>
#define ULIB_PAUSE_FUNCTIONS
#include "akc62a.h"

struct command_buffer  keypadCommand;

extern BOOLEAN	haveCartCode, resetReady;
extern bank1 BYTE	   theState;

bank1 BYTE access_code[4]={0,0,0,0},
           i2cSendBuffer[SEND_BUFFER_SIZE],
           zeroCount=0;

BYTE key[2][NUMBER_OF_KEYS] =
        {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; // raw key presses
bank1 BYTE   keyPressed[2] = {NUMBER_OF_KEYS,NUMBER_OF_KEYS};
bank1 BYTE   numKeysPressed[2] = {0,0};

extern bank1 BYTE amcCommand;
extern bank1 BYTE code2Send;
extern BYTE nextPop,nextOut;

/* --------------------------- AKC_RawReadKeypad --------------------------

 Purpose:
       Reads the keypad without any debouncing. The state of each key is
       stored in the array whose element 0 is pointed to by key_value

   ---------------------------------------------------------------------- */
BYTE AKC_RawReadKeypad(BYTE* theKeyValue,BYTE* theKeyPress)
{
int     k;
BYTE    returnValue;

  RA1=HIGH;
  RA2=HIGH;
  RA3=HIGH;
  RA0=LOW;
  mAKC_Settle();
  *(theKeyValue+POUND_KEY) = !RB0;
  *(theKeyValue+9) = !RB1;
  *(theKeyValue) = !RB2;
  RA0=HIGH;
  RA1=LOW;
  mAKC_Settle();
  *(theKeyValue+STAR_KEY) = !RB0;
  *(theKeyValue+4) = !RB1;
  *(theKeyValue+5) = !RB2;
  mAKC_Settle();
  RA1=HIGH;
  RA2=LOW;
  mAKC_Settle();
  *(theKeyValue+8) = !RB1;
  *(theKeyValue+7) = !RB3;
  *(theKeyValue+6) = !RB4;
  mAKC_Settle();
  RA2=HIGH;
  RA3=LOW;
  mAKC_Settle();
  *(theKeyValue+3) = !RB1;
  *(theKeyValue+2) = !RB3;
  *(theKeyValue+1) = !RB4;
  returnValue = 0;
  *theKeyPress = NUMBER_OF_KEYS;
  for (k = 0; k < NUMBER_OF_KEYS; k++)
  {
    if (*(theKeyValue+k))
    {
      *theKeyPress = k;
      returnValue++;
    }
  }


  return returnValue;
}


/* ---------------------------- AKC_Keysequal ----------------------------


   ---------------------------------------------------------------------- */
int AKC_Keysequal()
{
    return (numKeysPressed[0] == numKeysPressed[1]) && (keyPressed[0] == keyPressed[1]);
}

/* ---------------------------- AKC_ReadKeypad ----------------------------

       Debounces the keypad and stores the state of each key in the
       key_stroke array.

   ---------------------------------------------------------------------- */
int AKC_ReadKeypad()
{

int previous = 0;

  numKeysPressed[previous] =
      AKC_RawReadKeypad(&(key[previous][0]),&(keyPressed[previous]));
  if (numKeysPressed[previous])
  {
    do
    {
      previous = !previous;
      if (!AKC_Keysequal())
 	      AKC_Wait(DEBOUNCE_DELAY);
		  else
		    break;

     }
     while ( numKeysPressed[previous] =
	            AKC_RawReadKeypad(&(key[previous][0]),&(keyPressed[previous])));
     AKC_Wait(DEBOUNCE_DELAY*5);  /* this is just to reduce key repeats */
     return (numKeysPressed[0]);
  }
  return 0;

}

/* --------------------------- AKC_ReadKeyNum ----------------------------


   ---------------------------------------------------------------------- */
int AKC_ReadKeyNum()
{
  switch (AKC_ReadKeypad())
  {
    case 2: if ( key[0][1] && key[0][2] )
                return NUMBER_OF_KEYS+2;
            else
                return keyPressed[0];
    case 0:
	    return NUMBER_OF_KEYS + 1;
    default:
	    return keyPressed[0];
  }
}

/* ----------------------------- AKC_ReadKey -----------------------------


   ---------------------------------------------------------------------- */
char AKC_ReadKey()
{
int k;

  k = AKC_ReadKeyNum();
  if (k > NUMBER_OF_KEYS)
    return ('~');
  else
  	{
	   AKC_Blip();
	   switch (k)
    {
      case STAR_KEY: return ('*');
      case POUND_KEY: return ('#');
      default:    return ('0' + k);
    }
	 }
}


/* --------------------------- AKC_AccumKeypad ----------------------------

      Processes a stream of characters from the keypad and parses it into
      commands and then calls the code needed to execute the commands.

      Keypad commands consist of a user id followed by '*' followed by
      the user's PIN number followed by '*' followed by the cart access
      code followed by '*' followed by a 1-digit drawer number. If the
      access code is correct the drawer will pop and the access will be
      logged. Also any bins opened will be recorded. Each single digit
      typed thereafter will open a drawer until a '#' is entered. Then a
      new user id must be entered.

      ONLY access code and drawer pops are currently implemented  !!
   ---------------------------------------------------------------------- */
void AKC_AccumKeypad(char c)
{
int i;

   if ( (resetReady==TRUE) && (c == '*' ) )
     AKC_Reboot();           /* reset akc & main controller */

   resetReady=FALSE;
   if ( c == '#' )
   {
      mAKC_SignalAMCReset();       /* turn off the signal to the AMC */
      SSPOV = FALSE;             /* clear the I2C data buffer overflow flag */
      keypadCommand.count =0;    /* reset keypad buffer counter */
      code2Send=0;              /* reset outstanding data-to-send-to-AMC counter */
		nextOut=0;                /* reset the I2C transmit circular buffer counter */
		nextPop=0;                 /* reset character mode pop-drawer circular buffer counter */
      haveCartCode=FALSE;        /* reset the access code tracking flag */
      mAKC_ForceState(IDLE_STATE); /* set to only current state */
      return;
   }

   if (haveCartCode) /* after code is validated, all numbers pop a drawer if exists */
	{
      if ( (c >= '0') && (c<='9') )
        AKC_PopDrawer(c-0x30); /* convert character code to drawer index */
      else if (c == '*')
      {
        AKC_PopDrawer(0);     /* dummy drawer pop to wakeup scb */
        AKC_Wait(50);
        mAKC_SignalAMCReset();
      }
      return;                  /* before sending to be popped */
   }

   if (( c != '*' ) && (keypadCommand.count < SEND_BUFFER_SIZE) )
   {
      keypadCommand.buffer[keypadCommand.count++] = c;
      code2Send++;
   }
   else if ( c == '*' ) /* end of access code entry */
   {
      if (code2Send==0)       /* if no access code digits entered yet, then treat as */
      {
        AKC_PopDrawer(-1);     /* dummy drawer pop to wakeup scb */
        AKC_Wait(50);
        mAKC_SignalAMCReset();
      }
      else
      {
        i2cSendBuffer[0] = CODE_VERIFY + 0x0A;
        i2cSendBuffer[1] = keypadCommand.count;
        code2Send = 2;       /* extra 2 data bytes at front are opcode & data length byte */
        for (i=0; i<keypadCommand.count; i++,code2Send++)
          i2cSendBuffer[i+2] = keypadCommand.buffer[i];
        mAKC_SetState(WAITING_STATE);
        haveCartCode = TRUE;
        nextOut=0;
        mAKC_SignalAMC();
      }
   } /* end - if c == '*' */
   else     /* too many keys have come in now to be valid so reset everything */
   {
      mAKC_ClearState(WAITING_STATE);
      keypadCommand.count=0;
      code2Send=0;
      haveCartCode=FALSE;
      nextOut=0;
      mAKC_SignalAMCReset();
   }
}

/* -------------------------- AKC_ProcessKeypad ---------------------------

      reads and processes a key press, if any.

   ---------------------------------------------------------------------- */

void AKC_ProcessKeypad()
{
char c;
   c = AKC_ReadKey();     // returns '~' if no key press
   if (c != '~')
   {
      AKC_Wait(50);
      AKC_AccumKeypad(c);
   }
}

/* ------------------------------ AKC_Beep -------------------------------


   ---------------------------------------------------------------------- */
void AKC_Beep()
{
   READY_LED_PIN = !READY_LED_PIN;
   BUZZER_PIN = LOW;
   AKC_Wait(BEEP_DURATION);
   BUZZER_PIN = HIGH;
   READY_LED_PIN = !READY_LED_PIN;
}

/* ------------------------------ AKC_Blip -------------------------------


   ---------------------------------------------------------------------- */
void AKC_Blip()
{
	READY_LED_PIN = !READY_LED_PIN;
	BUZZER_PIN = LOW;
   AKC_Wait(BLIP_DURATION);
	BUZZER_PIN = HIGH;
	READY_LED_PIN = !READY_LED_PIN;
}


/* ------------------------- AKC_CartActiveAlert -------------------------


   ---------------------------------------------------------------------- */
void AKC_CartActiveAlert()
{
int j;

  for(j=0;j<CART_ACTIVE_BEEPS;j++)
  {
     READY_LED_PIN=!READY_LED_PIN;
	  BUZZER_PIN = LOW;
	  AKC_Wait(ALERT_BEEP_DURATION);
     READY_LED_PIN = !READY_LED_PIN;
	  BUZZER_PIN = HIGH;
	  AKC_Wait(ALERT_BEEP_DURATION);
  }
}

/* ---------------------- AKC_DrawerAlreadyOpenAlert ---------------------


   ---------------------------------------------------------------------- */
void AKC_DrawerAlreadyOpenAlert()
{
int j;

  for(j=0;j<ALREADY_OPEN_BEEPS;j++)
  {
    OPEN_LED_PIN = !OPEN_LED_PIN;
    BUZZER_PIN = LOW;
    AKC_Wait(ALERT_BEEP_DURATION);
    OPEN_LED_PIN = !OPEN_LED_PIN;
    BUZZER_PIN = HIGH;
    AKC_Wait(ALERT_BEEP_DURATION);
  }
}

/* --------------------------- AKC_PopDrawer ----------------------------


   ---------------------------------------------------------------------- */
void AKC_PopDrawer(char drawer)
{
  if (drawer == 0)
  {
    if (++zeroCount == 6)
    {
      zeroCount=0;
      resetReady=TRUE;
    }
    return;
  }
  if (mAKC_AMCIsSignalled)
    return;
  OPEN_LED_PIN = !OPEN_LED_PIN;
  amcCommand = OPEN_DRAWER + drawer;
  i2cSendBuffer[ nextPop++] = OPEN_DRAWER + drawer;
  if (nextPop >= SEND_BUFFER_SIZE)
    nextPop=0;
  mAKC_SetState(WAITING_STATE);
  mAKC_SignalAMC();
    AKC_Wait(5);
  OPEN_LED_PIN = !OPEN_LED_PIN;
}

/* --------------------------- AKC_CodeIsValidBuzz -----------------------


   ---------------------------------------------------------------------- */
void AKC_CodeIsValidBuzz()
{
BYTE i;

  READY_LED_PIN=!READY_LED_PIN;
  for(i=0;i<CODE_VALID_BUZZ_DURATION;i++)
  {

    AKC_Blip();
    AKC_Wait(3);

  }
  READY_LED_PIN=!READY_LED_PIN;

}


