///////////////// cart.h /////////////////////////////
// cart controller including smart drawer commands
//////////////////////////////////////////////////////

/* The TDE 4 doesn't support the 877, we don't get all the PROM memory we */
/* should, so we have to compile for the 77, but need to save some memory to do that, so ... */
/* this can be disabled in TechTools by adding -DTDE to the Setup:Project dialog, in the */
/* options box */


#include <pic.h>
#ifndef _CART_H_
#define _CART_H_


#define ULIB_ALL_FUNCTIONS
#include "\ulib\ulib.h"   /* serial, delay & I2C utilities */

/* ----------------------------- DEFINES ---------------------------------- */


#define USE_DIAGNOSTICS 1    /* set to 0 for more program memory */
#define USE_TEST        1    /* set to 0 for more program memory */
#define USE_RING_BUFFER 1

#define DIAGNOSTIC_SWITCH_MASK  0x30  /* for interpreting the DIP switches */



#ifndef BPS
#define BPS	9600  /* serial baud rate can be overridden with -D directive */
#endif

/* ------------------------------------------------------------------------
   error status flags - these are not currently used, since error reporting
   is kept to a minimum due to program memory space considerations. They will
   be usefuly later.
   ------------------------------------------------------------------------ */
#define ERROR_DRAWER  0x01
#define ERROR_SMART_DRAWER 0x02
#define ERROR_LOGGING 0x02
#define ERROR_ACCESS  0x04
#define ERROR_COMMAND 0x08
#define ERROR_KEYPAD  0x10
#define ERROR_SCB     0x20
#define ERROR_EEPROM  0x40
#define ERROR_RTC     0x80

#define OFF			      0
#define ON	 		      1
#define ENABLED			1
#define DISABLED			0
#define SET    		  	1
#define CLEAR  		  	0
#define LOW             0
#define HIGH            1

#define ASSERTED      0
#define NOT_ASSERTED  1

#define MAX_ACCESS_CODE_LENGTH 4

#define AKC_ALERT_RETRIES      3
#define AKC_CHECK_PERIOD       10000
#define HEARTBEAT_PULSE_TIME   6553
#define HEARTBEAT_LED_PIN	    RB1
#define KPC_REQUEST         	 RB0


#define  LED_ON	             0x40
#define  LED_OFF	           0x50

#define KEYPAD_I2C_ADDRESS   0x7E
#define I2C_READ_BYTE_DELAY  30
#define ACCESS_CODE_LENGTH   4
#define KP_DRAWER_CMD_MASK   0xF0
#define KP_DRAWER_MASK       0x0F
#define KP_SAFETY_NIBBLE     0xA
#define KP_POP_DRAWER        0x1A
#define KP_CODE_REQUEST      0x2A
#define KP_DIAGNOSTIC        0x3A

#define MAX_RING_BUFFER_LENGTH 64

#define        BUFFER_SIZE 8             // maximum characters in the command buffer


/* ------------------------------ MACROS----------------------------------- */

#define mAMC_FlowinOff() mAMC_RTSOff
#define mAMC_FlowinOn()  mAMC_RTSOn

#define mAMC_TOGGLEHEARTBEATLED() TRISB1=!TRISB1

#define mAMC_SETDEFAULTACCESSCODE()   {\
    cart.accessCode[0] = '1';\
    cart.accessCode[1] = '0';\
    cart.accessCode[2] = '3';\
    cart.accessCode[3] = '6';\
	 }
#define mAMC_SETDEFAULTSCBCONNECTWAIT()   {\
    cart.scbConnectTimeout = 6;\
	}
#define mAMC_SETDEFAULTIDLEWAIT()   {\
    cart.sleepTimeout = 30;\
	}
#define mAMC_SETDEFAULTDRAWEROPENTIMEOUT()   {\
    cart.drawerOpenTimeout = 3;\
	}
#define mAMC_SETDEFAULTCONFIGURATION()  {\
	 mAMC_SETDEFAULTACCESSCODE();\
    mAMC_SETDEFAULTSCBCONNECTWAIT();\
    mAMC_SETDEFAULTIDLEWAIT();\
    mAMC_SETDEFAULTDRAWEROPENTIMEOUT();\
    cart.checksum = AMC_ChecksumValue();\
	 }

typedef struct command_buffer
{
  int count;
  char buffer[BUFFER_SIZE];
} CommandBuffer;


struct cart_configuration
{
  BYTE    accessCode[MAX_ACCESS_CODE_LENGTH];
  BYTE    sleepTimeout;      /* minutes of inactivity before turning off serial client bridge */
  BYTE    scbConnectTimeout;   /* seconds to wait when re-connecting scb before deciding server is unreachable */
             /* this value is multiplied by SCB_RECONNECT_FACTOR to decide how often to retry connecting */
  BYTE    drawerOpenTimeout;   /* minutes that drawer is open before alarm */
  BYTE    checksum;
  long    logCount;          /* number of entries currently in log */
};


extern struct cart_configuration cart;

#endif  /* _CART_H_ */

/* prototypes */
void AMC_ProcessCommand(void);
BYTE AMC_ParseCommand(char c);
