/* power.c */

#include <pic.h>

#include "amc.h"
#include "remoteio.h"
#include "timer.h"
#include "power.h"

extern BYTE    batteryState,
                     powerState,
                     lastBatteryState,
                     lastPowerState;
extern bank2 BYTE    volts;
extern bank1 BYTE    scbSleepTimeout;
extern BOOLEAN doPowerUpdate;
extern BOOLEAN forceSleep;
extern BOOLEAN sleepLEDstate;
extern BOOLEAN cartIsActive;

void
AMC_GetVoltage() {
  ADON=ENABLED;
  MicroPause(20);
  ADGO=ENABLED;
  while(ADGO)  continue;
  ADON=DISABLED;
#if defined (_16F877)
  volts=ADRESH;
#else
  volts=ADRES;
#endif
}

AMC_GetBatteryState()
{
   AMC_GetVoltage();
   if (volts > BATTERY_CHARGED_THRESHOLD)
     batteryState = BATTERY_CHARGED;
   else if  (volts > BATTERY_OK_THRESHOLD)
     batteryState = BATTERY_OK;
   else if  (volts > BATTERY_LOW_THRESHOLD)
     batteryState = BATTERY_LOW;
   else batteryState = BATTERY_DEAD;
}

void AMC_UpdatePower()
{
   doPowerUpdate = FALSE;

   if (scbStatus==SCB_OFFLINE)
   {
     AMC_KpLed(SLEEP_LED, LED_ON);
     AMC_BusyLoop(250);
     AMC_KpLed(SLEEP_LED, LED_OFF);
   }

   if (lastBatteryState == BATTERY_DEAD)
     AMC_SendAlert(BATTERY_DEAD_ALERT);

   powerState = mPOWER_STATUS;
   AMC_GetBatteryState();

   if (mAMC_PowerStateChanged)
   {
     switch(powerState)
     {
     case POWER_BAD:
         AMC_KpLed(CHARGING_LED, LED_ON);
         AMC_KpLed(LOW_BATT_LED, LED_ON);
         AMC_KpLed(SERVICE_LED, LED_ON);
       break;
     case POWER_MAINS:
         if (batteryState == BATTERY_CHARGED)
           AMC_KpLed(CHARGING_LED, LED_OFF);
         else
           AMC_KpLed(CHARGING_LED, LED_ON);
         if (cartIsActive)
           AMC_KpLed(SERVICE_LED, LED_OFF);
       break;
     case POWER_BATTERY:
         AMC_KpLed(CHARGING_LED, LED_OFF);
         if (cartIsActive)
           AMC_KpLed(SERVICE_LED, LED_OFF);
       break;
   }
  }
  MicroPause(100);

  /* NOTE: the following states are */
  if (mAMC_BatteryStateChanged)
  {
    forceSleep=FALSE;
    switch (batteryState)
    {
      case BATTERY_CHARGED:
          AMC_KpLed(LOW_BATT_LED, LED_OFF);
          AMC_KpLed(READY_LED, LED_ON);
          if (cartIsActive)
            AMC_KpLed(SERVICE_LED, LED_OFF);
          if (powerState==POWER_MAINS)
             AMC_KpLed(CHARGING_LED, LED_OFF);
          scbSleepTimeout = cart.sleepTimeout;

      case BATTERY_OK:
          scbSleepTimeout = cart.sleepTimeout > 1 ? cart.sleepTimeout/2 : 1;
        break;

      case BATTERY_LOW:
          AMC_KPBeep(BEEP);
          AMC_KpLed(LOW_BATT_LED, LED_ON);
          AMC_KpLed(READY_LED, LED_OFF);
          AMC_KpLed(SERVICE_LED, LED_OFF);
          scbSleepTimeout =  1;
       break;

      case BATTERY_DEAD:
      default:
          AMC_KPBeep(BUZZER_ON);
          AMC_KpLed(LOW_BATT_LED, LED_ON);
          AMC_KpLed(READY_LED, LED_OFF);
          AMC_KpLed(SERVICE_LED, LED_ON);
          MilliPause(1000);
          AMC_KPBeep(BUZZER_OFF);
          forceSleep = TRUE;
        break;
    }
   }
   AMC_ResetFastEvent(POWER_TIMEOUT_EVENT,POWER_UPDATE_PERIOD);
   lastBatteryState = batteryState;
   lastPowerState = powerState;
}
