/* power.h */

void AMC_UpdatePower(void);

/* serial client bridge stuff */

#define AMC_CTS			RA4
#define AMC_DCD		   RA5
#define AMC_DSR         RB2

#define AMC_RTS			RA2
#define mAMC_RTSOn		RA2=0
#define mAMC_RTSOff		RA2=1

#define mAMC_SCBActive  !RA4

#define SCB_LOAD_VOLTAGE_OFFSET  50

#define SCB_RECONNECT_FACTOR  3

#define SCB_ONLINE 0
#define SCB_SLEEPING  1
#define SCB_OFFLINE   2

extern  BYTE     scbStatus,
                      batteryStatus;

#define POWER_BAD        2
#define POWER_MAINS      1
#define POWER_BATTERY    0


#define BATTERY_CHARGED  3
#define BATTERY_OK       2
#define BATTERY_LOW      1
#define BATTERY_DEAD     0

#define BATTERY_CHARGED_THRESHOLD  0xC4  /* 196 - 12 volts */
#define BATTERY_OK_THRESHOLD       0xAC  /* 172 - 10.5 volts */
#define BATTERY_LOW_THRESHOLD      0x93  /* 147 - 9 volts */

#define POWER_UPDATE_PERIOD  10000   /* in milliseconds */

#define mPOWER_STATUS    !RE0

#define mAMC_PowerStateChanged ((lastPowerState != powerState) || (powerState == POWER_BAD))
#define mAMC_BatteryStateChanged ((lastBatteryState != batteryState) || (batteryState <= BATTERY_LOW))

 /* mAMC_GetVoltage()  */
 /* - enable ADC using Fosc/2, channel 0    */
 /* - wait 5 uSec before starting conversion */
 /* - wait for conversion to complete        */
 /* - turn off ADC                           */
 /* puts the voltage in 'volts' */
#define mAMC_GetVoltage() {\
  ADCON0=0x01;\
  MicroPause(5);\
  ADGO=ENABLED;\
  while(ADGO)  continue;\
  ADCON0 = 0x00;\
#if defined (_16F877)\
  volts = ADRESH;\
#else\
  volts=ADRES;\
#endif\
}


#define mAMC_GetBatteryState() {\
   AMC_GetVoltage();\
   if (volts > BATTERY_CHARGED_THRESHOLD) batteryState = BATTERY_CHARGED;\
   else if  (volts > BATTERY_OK_THRESHOLD) batteryState = BATTERY_OK;\
   else if  (volts > BATTERY_LOW_THRESHOLD) batteryState = BATTERY_LOW;\
   else batteryState = BATTERY_DEAD;\
}


