/* ulib.h */
/* By Kenneth C. Finney.  Copyright 1998,1999. All Rights reserved */
/* Rev. 2.201 (P) */

#include "\ulib\udefs.h"  /* useful (or needed) definitions */

#ifdef ULIB_ALL_FUNCTIONS
#define ULIB_SERIAL_FUNCTIONS
#define ULIB_I2C_FUNCTIONS
#define ULIB_PAUSE_FUNCTIONS
#endif

#define LOW 0
#define HIGH 1

#ifndef FALSE
  #define FALSE 0
  #define TRUE !FALSE
#endif

#define BYTE unsigned char
#define BOOLEAN bit


extern void           U_InitULib(unsigned long freq);

/* I2C library */

#ifdef ULIB_I2C_FUNCTIONS

extern BYTE i2cError;

#define I2C_NO_ERROR  0
#define I2C_CLOCK_NOT_RELEASED  1
#define I2C_ACKNOWLEDGE   2
#define I2C_NO_ACKNOWLDGE 3

/* I2C stuff */
/* prototypes */

extern void           U_I2CStart(void);
extern void           U_I2CStop(void);
extern signed char    U_I2CWrite(unsigned char);
extern char           U_I2CRead(unsigned char);


/*
 *  SDA (data) and SCL (clock) bits
 */

/* these defines are necessary to enable the KCF_I2C stuff */

#define MORE		TRUE
#define LAST		FALSE

#define SCL   		RC3   	/* clock on port C bit 3 */
#define SCL_DIR   TRISC3

#define SDA   		RC4   	/* data on port C bit 4 */
#define SDA_DIR   TRISC4

#define I2C_INPUT 1   /* data direction input */
#define I2C_OUTPUT  0   /* data direction output */

#define I2C_READ  0x01    /* read bit used with address */
#define I2C_WRITE 0x00    /* write bit used with address */

#define FALSE   0
#define TRUE    !FALSE

#define I2C_ERROR (-1)
#define I2C_LAST  FALSE   /* SendAck: no more bytes to send */
#define I2C_MORE  TRUE    /* SendAck: more bytes to send */

/*
 * Timings for the i2c bus. Times are rounded up to the nearest
 * micro second.
 */

#define I2C_TM_BUS_FREE   5
#define I2C_TM_START_SU   5
#define I2C_TM_START_HD   4
#define I2C_TM_SCL_LOW    5
#define I2C_TM_SCL_HIGH   8
#define I2C_TM_DATA_SU    1
#define I2C_TM_DATA_HD    0
#define I2C_TM_DATA_HD_WAIT ;
#define I2C_TM_SCL_TO_DATA  4                   /* SCL low to data valid */
#define I2C_TM_STOP_SU    4
#define I2C_TM_SCL_TMO    10                    /* clock time out */
#define I2C_TM_SCL_TMO_WAIT MicroPause(10)

#define I2C_CLOCK_LOW  {SCL=LOW;SCL_DIR=I2C_OUTPUT;}
#define I2C_DATA_LOW   {SDA=LOW;SDA_DIR=I2C_OUTPUT;}

#define I2C_CLOCK_HIGH SCL_DIR=I2C_INPUT;
#define I2C_DATA_HIGH  SDA_DIR=I2C_INPUT;

#define I2C_SEND_NACK {I2C_DATA_HIGH;I2C_SEND_CLOCK_PULSE;}
#define I2C_SEND_ACK {I2C_SEND_SDA_LOW;I2C_SEND_CLOCK_PULSE;}
#define I2C_SEND_CLOCK_PULSE {I2C_CLOCK_HIGH;MicroPause(TM_SCL_HIGH);I2C_CLOCK_LOW;}

#endif


/* serial port stuff */
/* note: XTAL_FREQ is defined in application header files or compiler command line */


#ifdef ULIB_SERIAL_FUNCTIONS

#define U_EIGHT_BITS	(0)
#define U_NINE_BITS	(1)

#define mU_ENABLE_SERIAL_PORT() SPEN = 1
#define mU_DISABLE_SERIAL_PORT() SPEN = 0

void putch(char c);
char getch(void);
void U_Puts(const char *string);
void U_Gets(char *string);
extern unsigned char	U_SerialInit(unsigned long int);
extern unsigned char	U_SerialCheckOERR(void);

//U_Printf(const char * f, ...);
void U_PrintHex(BYTE d);

#if defined(_16C74)	|| defined(_16C77) || defined(_16F877)
extern bank3 BYTE u_maxDecimal;
extern bank3 BYTE u_minDecimal;
extern bank3 BYTE u_maxWidth;
extern bank3 BOOLEAN u_leadingZero;
#else
extern bank1 BYTE u_maxDecimal;
extern bank1 BYTE u_minDecimal;
extern bank1 BYTE u_maxWidth;
extern bank1 BOOLEAN u_leadingZero;
#endif

#define mU_SET_MAX_DECIMAL(__a__) u_maxDecimal=__a__
#define mU_SET_MIN_DECIMAL(__a__) u_minDecimal=__a__
#define mU_SET_MAX_WIDTH(__a__) u_maxWidth=__a__
#define mU_SET_LEADING_ZERO(__a__) u_leadingZero=__a__

void U_PrintDec(BYTE num);
void U_PrintFlag(BYTE d);

#define mU_KBHIT() RCIF
#endif


/*----------------------------------Pause code stuff ------------------------*/
#ifdef ULIB_PAUSE_FUNCTIONS

#define KICK_DOG  1              /* clear the watchdog timer in MilliPause */
#define PAT_DOG   0              /* don't clear the watchdog timer in MilliPause */
#define LET_SLEEPING_DOGS_LIE 0  /* what it says */

#define SPEED_CONSTANT  0.712/ (20000000L/(XTL*1000000))

#ifndef SML8
#if XTL > 9
#define MicroPause(x) {\
 if (x>1) {unsigned int tcount = x * SPEED_CONSTANT; \
   while(--tcount) continue; \
   if (x < 10 ) asm("nop");\
 }\
 else {\
  asm("nop");asm("nop");\
 }\
}

#define PAUSE400ns   asm("nop");
#define PAUSE800ns   asm("nop");asm("nop");
#define PAUSE1200ns  PAUSE800ns;PAUSE400ns;
#define PAUSE1600ns  PAUSE800ns;PAUSE800ns;
#define PAUSE2000ns  PAUSE800ns;PAUSE800ns;PAUSE400ns;

#else
#define MicroPause(x) {\
 if (x>1) {unsigned int tcount = x * SPEED_CONSTANT; \
   while(--tcount) continue; \
 }\
}
#define PAUSE1600ns  asm("nop");
#define PAUSE2400ns  asm("nop");asm("nop");
#endif
#else
#define MicroPause(x)
#define PAUSE400ns
#define PAUSE800ns
#define PAUSE1200ns
#define PAUSE1600ns
#define PAUSE2000ns
#define PAUSE2400ns
#endif

#if USE_WDT
extern void MilliPause(unsigned int cnt, BYTE kickDog);
extern void ISRMilliPause(unsigned int cnt, BYTE kickDog);
#else
extern void MilliPause(unsigned int cnt);
extern void ISRMilliPause(unsigned int cnt);
#endif
#endif


