/* kcf.c */
/* all support functions required for newcart project */
/* with the HIGHT-TECH C Compiler */

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include <pic.h>

#define I2C_DEFINITIONS

#include "kcf.h"

/*
 *	Delay functions for the PIC
 *
 *	Functions available:
 *		DelayUs(x)	Delay specified number of microseconds
 *		DelayMs(x)	Delay specified number of milliseconds
 *
 *	Note that there are range limits: x in Ms must not exceed 16535 - for xtal
 *	frequencies > 12MHz the range for DelayUs is even smaller.
 *	To use DelayUs it is only necessary to include this file; to use
 *	DelayMs you must include delay.c in your project.
 *
 */

/*	Set the crystal frequency in the CPP predefined symbols list in
	HPDPIC, or on the PICC commmand line, e.g.
	picc -DXTAL_FREQ=4000000

	or
	picc -DXTAL_FREQ=100000

	Note that this is the crystal frequency, the CPU clock is
	divided by 4.

 *	MAKE SURE this code is compiled with full optimization!!!

 */
void
KCF_DelayMs(unsigned int cnt)
{
#if	XTAL_FREQ <= 2000000
	do
   {
		KCF_DelayUs(995);/* the code chews up 5 us on each call */
	} while(--cnt);
#endif

#if    XTAL_FREQ > 2000000
unsigned char	i;
	do
   {
		i = 4;
		do
      {
			KCF_DelayUs(245); /* the code chews up 5 us on each call */
		}
      while(--i);
	}
   while(--cnt);
#endif
}

#ifdef USE_ISR_DELAY
unsigned int gMsDelay;
void
KCF_ISRDelayMs()
{
#if	XTAL_FREQ <= 2000000
	do
   {
		KCF_DelayUs(995);/* the code chews up 5 us on each call */
	} while(--gMsDelay);
#endif

#if    XTAL_FREQ > 2000000
unsigned char	i;
	do
   {
		i = 4;
		do
      {
			KCF_DelayUs(245); /* the code chews up 5 us on each call */
		}
      while(--i);
	}
   while(--gMsDelay);
#endif
}
#endif /* USE_ISR_DELAY */


void
KCF_I2CStop(void)
{
  /* assume SCL is high on entry */

  I2C_SEND_SCL_LOW;                  /* ensure clock is low */
  I2C_SEND_SDA_LOW;						/* data starts out low */
  I2C_SEND_SCL_HIGH;                 /* clock needs to be high */
  I2C_SEND_SDA_HIGH;                 /* low-high transition */

  return;
}

/*
 *  Send start condition
 *    - ensure data is high then issue a start condition
 *    - see also i2c_Start() macro
 */

void
KCF_I2CStart(void)
{
   I2C_SEND_SCL_LOW;                  /* ensure clock is low */
	I2C_SEND_SDA_HIGH;                 /* ensure data is high */
	I2C_SEND_SCL_HIGH;                 /* clock pulse high */
   I2C_SEND_SDA_LOW;                 /* the high->low transition */

  return;
}


/*
 *  Send a byte to the slave and acknowledges the transfer
 *    - returns I2C_ERROR, ack or ~ack
 */

signed char
KCF_I2CWrite(unsigned char data)
{
char i=8;
  unsigned char ack;

  while(i--)
  {
      SCL=LOW;
      SCL_DIR=I2C_OUTPUT;                 /* data hold time = 0, send data now */
		SDA =  ((data >> i)  & 0x01);	/* bit to send */
		SDA_DIR = I2C_OUTPUT;
      SCL_DIR=I2C_INPUT;
  }
  KCF_I2CWaitForSCL();
  if( i2cClockReleased==FALSE)		/* wait for clock release */
	return I2C_ERROR;	             	/* bus error */

  KCF_DELAY_4_US;

  SCL_DIR = I2C_OUTPUT;

  SDA_DIR = I2C_INPUT;      /* disable data line - listen for ack */

  KCF_DELAY_2_US;    /* SCL low to data out valid */
  SCL_DIR = I2C_INPUT;      /* float clock high */

  KCF_DELAY_2_US;

  ack = SDA;        /* read the acknowledge */

  /* wait for slave to release clock line after processing byte */
  KCF_I2CWaitForSCL();
  if( i2cClockReleased==FALSE)		/* wait for clock release */
    return I2C_ERROR;	               	/* bus error */

  return ack;
}

/*
 *  Get a byte from the slave and acknowledges the transfer
 *    - returns true on I2C_ERROR or byte
 */

char
KCF_I2CRead(unsigned char ack)
{
  unsigned char byt = 0;
  unsigned char i;

  for(i=0; i<8; i++)
  {
    SCL_DIR = I2C_OUTPUT;  /* float clock low */
    I2C_TM_SCL_LOW_WAIT;  /* min clock low  period */

    SDA_DIR = I2C_INPUT;    /* release data line */

    SCL_DIR = I2C_INPUT;    /* float clock high */

  i2cClockReleased=TRUE;
  if (SCL==LOW)
  {
 	 I2C_TM_SCL_TMO_WAIT;
	 if (SCL==LOW)
		 i2cClockReleased=FALSE;
  }
 if( i2cClockReleased==FALSE)		/* wait for clock release */
     return I2C_ERROR;	               	/* bus error */

	  //////////////
    I2C_TM_SCL_HIGH_WAIT;

    byt = byt << 1;   /* read the next bit */
    byt |= SDA;
  }

   SCL_DIR = I2C_OUTPUT;  /* float clock low */

  if (ack==I2C_MORE)
    SDA = LOW;     /* drive line low -> more to come */
  else
    SDA = HIGH;     /* drive line high -> no more wanted */

    SDA_DIR = I2C_OUTPUT;

    I2C_TM_DATA_SU_WAIT;

    SCL_DIR = I2C_INPUT;      /* float clock high */
    I2C_TM_SCL_HIGH_WAIT;

  return byt;
}


/* Routines for initialisation and use of the Serial Port
 * for the PIC 16C77 processor.
 */


unsigned char
KCF_SerInit(unsigned long int baud)
{
	int X;
	unsigned long tmp;

	/* calculate and set baud rate register */
	/* for asynchronous mode */
	tmp = 16UL * baud;
	X = (int)(XTAL_FREQ/tmp) - 1;
	if((X>255) || (X<0))
	{
		tmp = 64UL * baud;
		X = (int)(XTAL_FREQ/tmp) - 1;
		if((X>255) || (X<0))
		{
			return 1;	/* panic - baud rate unobtainable */
		}
		else
			BRGH = 0;	/* low baud rate */
	}
	else
		BRGH = 1;	/* high baud rate */
	SPBRG = X;	/* set the baud rate */

	SYNC = 0;	/* asynchronous */
	SPEN = 1;	/* enable serial port pins */
	CREN = 1;	/* enable reception */
	SREN = 0;	/* no effect */
	TXIE = 0;	/* disable tx interrupts */
	RCIE = 0;	/* disable rx interrupts */
	TX9  = 0;	/* 8-bit transmission */
	RX9  = 0;	/* 8-bit reception */
	TXEN = 1;	/* enable the transmitter */

	return 0;
}

void
putch(char c)
//KCF_SerPutByte(unsigned char byte)
{

  KCF_DelayMs(1);

  while(!TRMT)	/* set when register is empty */
    continue;
  TXREG = c;
   /*
	while(!TXIF)
		continue;
	TXREG = 0;
	*/

  return;
}

char
getch(void)
//KCF_SerGetByte(void)
{
	while(!RCIF)	/* set when register is not empty */
		continue;
	return RCREG;	/* RXD9 and FERR are gone now */
}


unsigned char
KCF_SerCheckOERR(void)
{
	if(OERR)	/* re-enable after overrun error */
	{
		CREN = 0;
		CREN = 1;
		return 1;
	}

	return 0;
}

void
puts(const char *string)
{
  while (*string)
  {
  	 putch(*string);
	 string++;
  }
  putch('\0');
}

void
gets(char *string)
{
char c;

  while ( (c = getch()) != '\r' )
  	 *string++ = c;
}


KCF_Printf(const char * f, ...)
{
char		c;
va_list		ap;
unsigned char	sign;
unsigned char	prec;
unsigned int	i;
signed char	width;

	va_start(ap, f);
	while(c	= *f++)
		if(c !=	'%')
			putch(c);
		else
		{
			prec = 255;
			width = 0;
			sign = 0;
loop:
			switch(c = *f++)
			{
			case 0:
				return;
#if 0
			case '*':
				width = va_arg(ap, int);
				goto loop;
#endif

#ifdef KCF_PRINTF_LONG
			case 'l':
				sign |= 0x80;
				goto loop;

#endif

			case 'd':
				sign++;
				goto decimal;
			case 'x':
#if 0
			case 'X':
#endif
				prec += 8;
#if 0
			case 'o':
#endif
				prec -= 2;
#if 0
			case 'u':
#endif

decimal:
				{
				unsigned int	j;

#ifdef KCF_PRINTF_LONG
					if(sign & 0x80)
						i = va_arg(ap, unsigned int);
					else
#endif
						i = va_arg(ap, int);
					if(sign & 1)
					{
					       if((int)i < 0)
							 {
								i = -i;
								width--;
					       }
							 else
						       sign = 0;
					}
					else
					{
#ifdef KCF_PRINTF_LONG
						if(!(sign & 0x80))
							i = (unsigned short)i;
#endif
						sign = 0;
					}
					prec -= 255-10;

					c = (unsigned int)i % prec;
					*(unsigned int *)&i /= prec;
					j = 1;
					while(j <= i)
					{
						j *= prec;
						width--;
					}
					while(--width > 0)
						putch(' ');
					if(sign & 1)
						putch('-');
					while(j /= prec)
					{
						width = (i/j)%prec;
						if(width > 9)
							width += 'A'-'0'-10;
						putch(width+'0');
					}
					if(c > 9)
						c += 'A'-'0'-10;
					putch(c+'0');
				}
				break;

			case 'c':
				while(width > 1)
				{
					putch(' ');
					width--;
				}
				c = va_arg(ap, int);
				putch(c);
				continue;

			case 's':
				{
					const char *	x;

					x = va_arg(ap, const char *);
					c = 0;
					while(x[c])
						c++;
					if(c < prec)
						prec = c;
					while(width > prec) {
						putch(' ');
						width--;
					}
					while(prec--) {
						c = *x++;
						putch(c);
					}
					continue;
				}

#if 0
			case '.':
				if(*f == '*') {
					prec = va_arg(ap, int);
					f++;
				} else {
					prec = *f++ - '0';
					c = *f;
					if(c >= '0' && c <= '9') {
						prec = prec*10 + c - '0';
						f++;
					}
				}
				goto loop;
#endif

			default:
				if(c >= '0' && c <= '9') {
					width = width * 10 + c - '0';
					goto loop;
				}
				putch(c);
				continue;

			}
		}
}


