
#include "kcf_parm.h"

#ifndef _KCF_UTIL_
#define _KCF_UTIL_

  #define LOW 0
  #define HIGH 1

  #ifndef FALSE
    #define FALSE 0
    #define TRUE !FALSE
  #endif

  #define BYTE unsigned char
  #define BOOLEAN bit

/*
 *	Delay functions for HI-TECH C on the PIC
 *
 *	Functions available:
 *		DelayUs(x)	Delay specified number of microseconds
 *		DelayMs(x)	Delay specified number of milliseconds
 *
 *	Note that there are range limits: x must not exceed 255 - for xtal
 *	frequencies > 12MHz the range for DelayUs is even smaller.
 *	To use DelayUs it is only necessary to include this file; to use
 *	DelayMs you must include delay.c in your project.
 *
 */

/*	Set the crystal frequency in the CPP predefined symbols list in
	HPDPIC, or on the PICC commmand line, e.g.
	picc -DXTAL_FREQ=4000000

	or
	picc -DXTAL_FREQ=100000

	Note that this is the crystal frequency, the CPU clock is
	divided by 4.

 *	MAKE SURE this code is compiled with full optimization!!!

 */

/* note: XTAL_FREQ is defined in kcf_parms.h */


#if	XTAL_FREQ >= 12000000L

#define	KCF_DelayUs(x)	{ unsigned char _dcnt; \
			  _dcnt = (x)*((XTAL_FREQ)/12000000L); \
			  while(--_dcnt != 0) \
				  continue; }
#else

#define	KCF_DelayUs(x)	{ unsigned char _dcnt; \
			  _dcnt = (x)/(12000000L/(XTAL_FREQ))|1; \
			  while(--_dcnt != 0) \
				  continue; }
#endif

#define KCF_DELAY_1_US   asm("nop");
#define KCF_DELAY_2_US   {KCF_DELAY_1_US;KCF_DELAY_1_US};
#define KCF_DELAY_3_US   {KCF_DELAY_2_US;KCF_DELAY_1_US};
#define KCF_DELAY_4_US   {KCF_DELAY_2_US;KCF_DELAY_2_US;}
#define KCF_DELAY_5_US   {KCF_DELAY_4_US;KCF_DELAY_1_US;}
#define KCF_DELAY_10_US  {KCF_DELAY_5_US;KCF_DELAY_5_US;}
#define KCF_DELAY_20_US  {KCF_DELAY_10_US;KCF_DELAY_10_US;}
#define KCF_DELAY_25_US  {KCF_DELAY_20_US;KCF_DELAY_5_US;}

extern void KCF_DelayMs(unsigned int);

#ifdef USE_ISR_DELAY
extern void KCF_ISRDelayMs();
extern unsigned int gMsDelay;
#endif

#ifdef I2C_DEFINITIONS
BOOLEAN i2cClockReleased=FALSE;
#else
extern BOOLEAN i2cClockReleased;
#endif

/* I2C stuff */
/* prototypes */

extern void           KCF_I2CStart(void);
extern void           KCF_I2CStop(void);
extern signed char    KCF_I2CWrite(unsigned char);
extern char           KCF_I2CRead(unsigned char);

#define KCF_I2CWaitForSCL()  { SCL_DIR=I2C_INPUT; \
                             if (SCL==LOW) { I2C_TM_SCL_TMO_WAIT; \
                               if (SCL==LOW)  SCL_DIR = I2C_OUTPUT; i2cClockReleased=FALSE; }\
									  SCL_DIR=I2C_OUTPUT; i2cClockReleased=TRUE;  }

/*
 *  SDA (data) and SCL (clock) bits
 */

/* these defines are necessary to enable the KCF_I2C stuff */

#define MORE		TRUE
#define LAST		FALSE

#define SCL   		RC3   	/* clock on port C bit 3 */
#define SCL_DIR   TRISC3

#define SDA   		RC4   	/* data on port C bit 4 */
#define SDA_DIR   TRISC4

#define I2C_INPUT 1   /* data direction input */
#define I2C_OUTPUT  0   /* data direction output */

#define I2C_READ  0x01    /* read bit used with address */
#define I2C_WRITE 0x00    /* write bit used with address */

#define FALSE   0
#define TRUE    !FALSE

#define I2C_ERROR (-1)
#define I2C_LAST  FALSE   /* SendAck: no more bytes to send */
#define I2C_MORE  TRUE    /* SendAck: more bytes to send */

/*
 * Timings for the i2c bus. Times are rounded up to the nearest
 * micro second.
 */

#define I2C_TM_BUS_FREE   5
#define I2C_TM_BUS_FREE_WAIT KCF_DELAY_5_US
#define I2C_TM_START_SU   5
#define I2C_TM_START_SU_WAIT KCF_DELAY_5_US
#define I2C_TM_START_HD   4
#define I2C_TM_START_HD_WAIT KCF_DELAY_4_US
#define I2C_TM_SCL_LOW    5
#define I2C_TM_SCL_LOW_WAIT KCF_DELAY_5_US
#define I2C_TM_SCL_HIGH   4
#define I2C_TM_SCL_HIGH_WAIT KCF_DELAY_4_US
#define I2C_TM_DATA_SU    1
#define I2C_TM_DATA_SU_WAIT KCF_DELAY_1_US
#define I2C_TM_DATA_HD    0
#define I2C_TM_DATA_HD_WAIT ;
#define I2C_TM_SCL_TO_DATA  4                   /* SCL low to data valid */
#define I2C_TM_SCL_TO_DATA_WAIT KCF_DELAY_4_US
#define I2C_TM_STOP_SU    4
#define I2C_TM_STOP_SU_WAIT KCF_DELAY_4_US
#define I2C_TM_SCL_TMO    10                    /* clock time out */
#define I2C_TM_SCL_TMO_WAIT KCF_DELAY_10_US

#define I2C_SEND_SCL_LOW  {SCL=LOW;SCL_DIR=I2C_OUTPUT;KCF_DELAY_5_US;}
#define I2C_SEND_SDA_LOW  {SDA=LOW;SDA_DIR=I2C_OUTPUT;KCF_DELAY_5_US;}

#define I2C_SEND_SCL_HIGH {SCL_DIR=I2C_INPUT;KCF_DELAY_5_US;}
#define I2C_SEND_SDA_HIGH {SDA_DIR=I2C_INPUT;KCF_DELAY_5_US;}

#define I2C_SEND_NACK {I2C_SEND_SDA_HIGH;I2C_SEND_CLOCK_PULSE;}
#define I2C_SEND_ACK {I2C_SEND_SDA_LOW;I2C_SEND_CLOCK_PULSE;}
#define I2C_SEND_CLOCK_PULSE {I2C_SEND_SCL_HIGH;I2C_SEND_SCL_LOW;}

/* serial port stuff */
/* note: XTAL_FREQ is defined in kcf_parms.h */

#define KCF_EIGHT_BITS	(0)
#define KCF_NINE_BITS	(1)

void putch(char c);
char getch(void);
void puts(const char *string);
void gets(char *string);

extern unsigned char	KCF_SerInit(unsigned long int);
//extern void 		KCF_SerPutByte(unsigned char);
//extern unsigned char	KCF_SerGetByte(void);
extern void 		KCF_SerPutNinth(unsigned char);
extern unsigned char	KCF_SerGetNinth(void);
extern unsigned char	KCF_SerGetFERR(void);
extern unsigned char	KCF_SerCheckOERR(void);
void   KCF_PutString(char *string);

KCF_Printf(const char * f, ...);

#define kbhit() RCIF


#endif

